#!/usr/bin/env nix-shell
#!nix-shell -i python -p python3


import os
import sys
import time
import struct
from socket import *


if len(sys.argv) < 2:
  print(f'Usage: {sys.argv[0]} HOST [STARTIDX]')
  exit(1)


port = 4213
host = sys.argv[1]
print(f'Using host[ {host} ]')


startIdx = 0
if len(sys.argv) > 2: startIdx = int(sys.argv[2])
print(f'Using startIdx[ {startIdx} ]')


def sendRaw(data: bytes, _socket = socket(AF_INET, SOCK_DGRAM)):
  try:
    _socket.sendto(data, (host, port))
  except:
    pass


class RGB:
  def __init__(self, r: int = 0, g: int = 0, b: int = 0):
    self.r = int(r)
    self.g = int(g)
    self.b = int(b)

  def __mul__(self, other: float):
    other = float(other)

    return RGB (
      self.r * other,
      self.g * other,
      self.b * other
    )

  def __rmul__(self, other: float):
    return self.__mul__(other)

  def __imul__(self, other: float):
    other = float(other)

    self.r = int(self.r * other)
    self.g = int(self.g * other)
    self.b = int(self.b * other)

    return self

  def toBytes(self):
    return struct.pack('BBB', self.r, self.g, self.b)

  def __repr__(self):
    return f'RGB({self.r}, {self.g}, {self.b})'


if 'USE_INJECTOR' in os.environ or '--use-injector' in sys.argv:
  from CyanBusInjector import CyanBusInjector

  cbInjector = CyanBusInjector(host, port)

  def _inject(data): cbInjector.sendPacket(payload=data)
  sendRaw = _inject


def sendPacket(startIdx: int = startIdx, pixel: list = [], header: bytes = b'fxCyanRGB8'):
  data = header + struct.pack('>H', startIdx)

  for px in pixel:
    data += px.toBytes()

  print(f'Sending |pixel|[ {len(pixel)} ]…')
  sendRaw(data=data)


if __name__ == '__main__':
  white = RGB(255, 255, 255)
  black = RGB(  0,   0,   0)
  red   = RGB(255,   0,   0)
  green = RGB(  0, 255,   0)
  blue  = RGB(  0,   0, 255)

  yellow = RGB(255, 255,   0)
  cyan   = RGB(0,   255, 255)
  violet = RGB(255,   0, 255)

  pixels = [
    red,
    RGB(255, 80, 0),
    yellow,
    RGB(80, 255, 0),
    green,
    cyan,
    blue,
    violet
  ]

  pixels = [ px * 0.2 for px in pixels ]

  FullHour = [
    red,
    green,
    blue
  ]

  sendPacket(pixel=[black]*8)

  fh = 0
  while True:
    sendPacket(pixel=pixels[:2])
    # sendPacket(pixel=pixels + [FullHour[fh]] + [black, black])

    fh += 1
    if fh >= len(FullHour): fh = 0

    time.sleep(0.25)
